import json
import subprocess

import requests

##
# Note!! This script should be run inside the public EN, since no public EN opens the governance namespace.
##

# rpc setup
PUBLIC_KEN_PATH="/usr/bin/ken"
PUBLIC_IPC_PATH="/var/kend/data/klay.ipc"
TEST_NODE_RPC = "http://34.1.198.150:8551"

method = "governance_getRewardsAccumulated"


def call_post_info(node_url, method, num):
    headers = {
        'Content-Type': 'application/json',
    }

    # RPC call data
    data = {
        "jsonrpc": "2.0",
        "method": method,
        # "params": [num],
        "params": [num, num+2],
        "id": 1
    }

    try:
        response = requests.post(node_url, headers=headers, data=json.dumps(data))
        response_data = response.json()

        if 'result' in response_data:
            result = response_data['result']
            return result
        else:
            return f"Error: {response_data.get('error', 'Unknown error')}"

    except Exception as e:
        return str(e)


def call_subprocess_get_info(ken_path, ipc_path, method, num):
    cmd = [
        ken_path,
        "--exec",
        f"{method.replace('_', '.')}({num}, {num+2})",
        "attach",
        ipc_path
    ]

    try:
        result = subprocess.check_output(cmd, text=True).strip()
        result = (
            result.replace("\n", "")  # 모든 개행 제거
            .replace("  ", " ")
            .replace("firstBlock:", '"firstBlock:')
            .replace("firstBlockTime", '"firstBlockTime')
            .replace("lastBlock:", '"lastBlock:')
            .replace("lastBlockTime", '"lastBlockTime')
            .replace("rewards", '"rewards')
            .replace("totalBurntTxFee", '"totalBurntTxFee')
            .replace("totalKEFRewards", '"totalKEFRewards')
            .replace("totalKIFRewards", '"totalKIFRewards')
            .replace("totalMinted", '"totalMinted')
            .replace("totalProposerRewards", '"totalProposerRewards')
            .replace("totalStakingRewards", '"totalStakingRewards')
            .replace("totalTxFee", '"totalTxFee')
            .replace("0x", '"0x')
            .replace(": ", '": ')
        )
        return result
    except subprocess.CalledProcessError as e:
        return f"Error: {e.output}"
    except Exception as e:
        return str(e)


def fetch_and_compare_data(num):
    results = {}
    errors = {}

    try:
        result = call_subprocess_get_info(PUBLIC_KEN_PATH, PUBLIC_IPC_PATH, method, num)
        # result = call_post_info(PUBLIC_NODE_RPC, method, num)
        results["public"] = result
    except Exception as e:
        errors["public"] = str(e)

    try:
        result = call_post_info(TEST_NODE_RPC, method, num)
        results["test"] = result
    except Exception as e:
        errors["test"] = str(e)

    public_data = results.get('public')
    test_data = results.get('test')

    print(f"# BlockNum: {num}")
    if isinstance(public_data, str) and public_data.startswith("Error:"):
        print("Error detected in 'public', skipping comparison.")
    elif isinstance(test_data, str) and test_data.startswith("Error:"):
        print("Error detected in 'test', skipping comparison.")
    else:
        unmatched = 0
        test_fields = ['number', 'totalSupply', 'totalMinted', 'totalBurnt', 'burntFee', 'zeroBurn', 'deadBurn', 'kip103Burn', 'kip160Burn']
        for test_field in test_fields:
            if public_data.get(test_field) != test_data.get(test_field):
                unmatched += 1
                print(f"Unmatched: {test_field} not equal: {public_data.get(test_field)}, {test_data.get(test_field)}")
        print(f"Unmatched: {unmatched}")
    print('[kairos dev ]',public_data)
    print('[kairos test]',test_data)
    print()


fetch_and_compare_data(0)
fetch_and_compare_data(1)
fetch_and_compare_data(127)
fetch_and_compare_data(128)
fetch_and_compare_data(75373311)
fetch_and_compare_data(75373312)
fetch_and_compare_data(75373313)
fetch_and_compare_data(80295290)
fetch_and_compare_data(80295291)
fetch_and_compare_data(80295292)
fetch_and_compare_data(86513894)
fetch_and_compare_data(86513895)
fetch_and_compare_data(86513896)
fetch_and_compare_data(98347375)
fetch_and_compare_data(98347376)
fetch_and_compare_data(98347377)
fetch_and_compare_data(111736799)
fetch_and_compare_data(111736800)
fetch_and_compare_data(111736801)
fetch_and_compare_data(119145599)
fetch_and_compare_data(119145600)
fetch_and_compare_data(119145601)
fetch_and_compare_data(131607999)
fetch_and_compare_data(131608000)
fetch_and_compare_data(131608001)
fetch_and_compare_data(141366999)
fetch_and_compare_data(141367000)
fetch_and_compare_data(141367001)
fetch_and_compare_data(156659999)
fetch_and_compare_data(156660000)
fetch_and_compare_data(156660001)
fetch_and_compare_data(156672001)
fetch_and_compare_data(162900479)
